Package: globaltest
Version: 5.66.0
Date: 2023-03-30
Title: Testing Groups of Covariates/Features for Association with a
        Response Variable, with Applications to Gene Set Testing
Author: Jelle Goeman and Jan Oosting, with contributions by Livio Finos, Aldo Solari, Dominic Edelmann
Maintainer: Jelle Goeman <j.j.goeman@lumc.nl>
Depends: methods, survival
Imports: Biobase, AnnotationDbi, annotate, graphics
Suggests: vsn, golubEsets, KEGGREST, hu6800.db, Rgraphviz, GO.db,
        lungExpression, org.Hs.eg.db, GSEABase, penalized, gss, MASS,
        boot, rpart, mstate
Description: The global test tests groups of covariates (or features) for association with a response variable. This package implements the test with diagnostic plots and multiple testing utilities, along with several functions to facilitate the use of this test for gene set testing of GO and KEGG terms.
License: GPL (>= 2)
biocViews: Microarray, OneChannel, Bioinformatics,
        DifferentialExpression, GO, Pathways
Collate: gt-object.R mlogit.R comparative.R diagnosticplots.R
        focuslevel.R generalizedFdistribution.R genesettesting.R gt.R
        inheritance.R models.R options.R utilities.R goodnessoffit.R
LazyLoad: yes
git_url: https://git.bioconductor.org/packages/globaltest
git_branch: RELEASE_3_23
git_last_commit: 8ca8e77
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 22:24:00 UTC; biocbuild
